/****************************************************************************
** Meta object code from reading C++ file 'kfileplacesview_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.28.0/src/filewidgets/kfileplacesview_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kfileplacesview_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN23KFilePlacesViewDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto KFilePlacesViewDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN23KFilePlacesViewDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KFilePlacesViewDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KFilePlacesViewDelegate, qt_meta_tag_ZN23KFilePlacesViewDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KFilePlacesViewDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemDelegate::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23KFilePlacesViewDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23KFilePlacesViewDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23KFilePlacesViewDelegateE_t>.metaTypes,
    nullptr
} };

void KFilePlacesViewDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KFilePlacesViewDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KFilePlacesViewDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KFilePlacesViewDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23KFilePlacesViewDelegateE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemDelegate::qt_metacast(_clname);
}

int KFilePlacesViewDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemDelegate::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN23KFilePlacesEventWatcherE_t {};
} // unnamed namespace

template <> constexpr inline auto KFilePlacesEventWatcher::qt_create_metaobjectdata<qt_meta_tag_ZN23KFilePlacesEventWatcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KFilePlacesEventWatcher",
        "entryMiddleClicked",
        "",
        "QModelIndex",
        "index",
        "headerAreaEntered",
        "headerAreaLeft",
        "actionEntered",
        "actionLeft",
        "actionClicked",
        "windowActivated",
        "windowDeactivated",
        "paletteChanged",
        "qScrollerStateChanged",
        "QScroller::State",
        "newState"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'entryMiddleClicked'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'headerAreaEntered'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'headerAreaLeft'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'actionEntered'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'actionLeft'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'actionClicked'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'windowActivated'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowDeactivated'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'paletteChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'qScrollerStateChanged'
        QtMocHelpers::SlotData<void(const QScroller::State)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KFilePlacesEventWatcher, qt_meta_tag_ZN23KFilePlacesEventWatcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KFilePlacesEventWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23KFilePlacesEventWatcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23KFilePlacesEventWatcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23KFilePlacesEventWatcherE_t>.metaTypes,
    nullptr
} };

void KFilePlacesEventWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KFilePlacesEventWatcher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->entryMiddleClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 1: _t->headerAreaEntered((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 2: _t->headerAreaLeft((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 3: _t->actionEntered((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 4: _t->actionLeft((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 5: _t->actionClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 6: _t->windowActivated(); break;
        case 7: _t->windowDeactivated(); break;
        case 8: _t->paletteChanged(); break;
        case 9: _t->qScrollerStateChanged((*reinterpret_cast<std::add_pointer_t<QScroller::State>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesEventWatcher::*)(const QModelIndex & )>(_a, &KFilePlacesEventWatcher::entryMiddleClicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesEventWatcher::*)(const QModelIndex & )>(_a, &KFilePlacesEventWatcher::headerAreaEntered, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesEventWatcher::*)(const QModelIndex & )>(_a, &KFilePlacesEventWatcher::headerAreaLeft, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesEventWatcher::*)(const QModelIndex & )>(_a, &KFilePlacesEventWatcher::actionEntered, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesEventWatcher::*)(const QModelIndex & )>(_a, &KFilePlacesEventWatcher::actionLeft, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesEventWatcher::*)(const QModelIndex & )>(_a, &KFilePlacesEventWatcher::actionClicked, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesEventWatcher::*)()>(_a, &KFilePlacesEventWatcher::windowActivated, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesEventWatcher::*)()>(_a, &KFilePlacesEventWatcher::windowDeactivated, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KFilePlacesEventWatcher::*)()>(_a, &KFilePlacesEventWatcher::paletteChanged, 8))
            return;
    }
}

const QMetaObject *KFilePlacesEventWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KFilePlacesEventWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23KFilePlacesEventWatcherE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KFilePlacesEventWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void KFilePlacesEventWatcher::entryMiddleClicked(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KFilePlacesEventWatcher::headerAreaEntered(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KFilePlacesEventWatcher::headerAreaLeft(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KFilePlacesEventWatcher::actionEntered(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KFilePlacesEventWatcher::actionLeft(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KFilePlacesEventWatcher::actionClicked(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KFilePlacesEventWatcher::windowActivated()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KFilePlacesEventWatcher::windowDeactivated()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void KFilePlacesEventWatcher::paletteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
