/****************************************************************************
** Meta object code from reading C++ file 'kfileitem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.28.0/src/core/kfileitem.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kfileitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9KFileItemE_t {};
} // unnamed namespace

template <> constexpr inline auto KFileItem::qt_create_metaobjectdata<qt_meta_tag_ZN9KFileItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KFileItem",
        "time",
        "",
        "KFileItem::FileTimes",
        "which",
        "timeString",
        "url",
        "QUrl",
        "user",
        "group",
        "isLink",
        "isDir",
        "isFile",
        "isReadable",
        "isWritable",
        "isHidden",
        "isSlow",
        "isDesktopFile",
        "linkDest",
        "targetUrl",
        "localPath",
        "isLocalFile",
        "text",
        "name",
        "mimetype",
        "determineMimeType",
        "QMimeType",
        "currentMimeType",
        "isFinalIconKnown",
        "isMimeTypeKnown",
        "mimeComment",
        "iconName",
        "overlays",
        "comment",
        "getStatusBarInfo",
        "isRegularFile",
        "FileTimes",
        "ModificationTime",
        "AccessTime",
        "CreationTime",
        "MimeTypeDetermination",
        "NormalMimeTypeDetermination",
        "SkipMimeTypeFromContent"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'time'
        QtMocHelpers::MethodData<QDateTime(KFileItem::FileTimes) const>(1, 2, QMC::AccessPublic, QMetaType::QDateTime, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Method 'timeString'
        QtMocHelpers::MethodData<QString(KFileItem::FileTimes) const>(5, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Method 'timeString'
        QtMocHelpers::MethodData<QString() const>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'url'
        QtMocHelpers::PropertyData<QUrl>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'user'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'group'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'isLink'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isDir'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isFile'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isReadable'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isWritable'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isHidden'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isSlow'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isDesktopFile'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'linkDest'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'targetUrl'
        QtMocHelpers::PropertyData<QUrl>(19, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'localPath'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'isLocalFile'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'text'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'mimetype'
        QtMocHelpers::PropertyData<QString>(24, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'determineMimeType'
        QtMocHelpers::PropertyData<QMimeType>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'currentMimeType'
        QtMocHelpers::PropertyData<QMimeType>(27, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'isFinalIconKnown'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isMimeTypeKnown'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'mimeComment'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'iconName'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'overlays'
        QtMocHelpers::PropertyData<QStringList>(32, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'comment'
        QtMocHelpers::PropertyData<QString>(33, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'getStatusBarInfo'
        QtMocHelpers::PropertyData<QString>(34, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'isRegularFile'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'FileTimes'
        QtMocHelpers::EnumData<enum FileTimes>(36, 36, QMC::EnumFlags{}).add({
            {   37, FileTimes::ModificationTime },
            {   38, FileTimes::AccessTime },
            {   39, FileTimes::CreationTime },
        }),
        // enum 'MimeTypeDetermination'
        QtMocHelpers::EnumData<enum MimeTypeDetermination>(40, 40, QMC::EnumFlags{}).add({
            {   41, MimeTypeDetermination::NormalMimeTypeDetermination },
            {   42, MimeTypeDetermination::SkipMimeTypeFromContent },
        }),
    };
    return QtMocHelpers::metaObjectData<KFileItem, qt_meta_tag_ZN9KFileItemE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KFileItem::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KFileItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KFileItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9KFileItemE_t>.metaTypes,
    nullptr
} };

void KFileItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<KFileItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDateTime _r = _t->time((*reinterpret_cast<std::add_pointer_t<KFileItem::FileTimes>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDateTime*>(_a[0]) = std::move(_r); }  break;
        case 1: { QString _r = _t->timeString((*reinterpret_cast<std::add_pointer_t<KFileItem::FileTimes>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { QString _r = _t->timeString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->url(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->user(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->group(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isLink(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isDir(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isFile(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isReadable(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isWritable(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->isHidden(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isSlow(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->isDesktopFile(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->linkDest(); break;
        case 12: *reinterpret_cast<QUrl*>(_v) = _t->targetUrl(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->localPath(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->isLocalFile(); break;
        case 15: *reinterpret_cast<QString*>(_v) = _t->text(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 17: *reinterpret_cast<QString*>(_v) = _t->mimetype(); break;
        case 18: *reinterpret_cast<QMimeType*>(_v) = _t->determineMimeType(); break;
        case 19: *reinterpret_cast<QMimeType*>(_v) = _t->currentMimeType(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->isFinalIconKnown(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->isMimeTypeKnown(); break;
        case 22: *reinterpret_cast<QString*>(_v) = _t->mimeComment(); break;
        case 23: *reinterpret_cast<QString*>(_v) = _t->iconName(); break;
        case 24: *reinterpret_cast<QStringList*>(_v) = _t->overlays(); break;
        case 25: *reinterpret_cast<QString*>(_v) = _t->comment(); break;
        case 26: *reinterpret_cast<QString*>(_v) = _t->getStatusBarInfo(); break;
        case 27: *reinterpret_cast<bool*>(_v) = _t->isRegularFile(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setUrl(*reinterpret_cast<QUrl*>(_v)); break;
        case 13: _t->setLocalPath(*reinterpret_cast<QString*>(_v)); break;
        case 16: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
