/********************************************************************************
** Form generated from reading UI file 'executablefileopendialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXECUTABLEFILEOPENDIALOG_H
#define UI_EXECUTABLEFILEOPENDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "kcontextualhelpbutton.h"

QT_BEGIN_NAMESPACE

class Ui_ExecutableFileOpenDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QLabel *iconLabel;
    QVBoxLayout *verticalLayout_2;
    QLabel *nameLabel;
    QLabel *mimeTypeLabel;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ExecutableFileOpenDialog)
    {
        if (ExecutableFileOpenDialog->objectName().isEmpty())
            ExecutableFileOpenDialog->setObjectName("ExecutableFileOpenDialog");
        ExecutableFileOpenDialog->resize(276, 152);
        verticalLayout = new QVBoxLayout(ExecutableFileOpenDialog);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(ExecutableFileOpenDialog);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);

        verticalLayout->addWidget(label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        iconLabel = new QLabel(ExecutableFileOpenDialog);
        iconLabel->setObjectName("iconLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(iconLabel->sizePolicy().hasHeightForWidth());
        iconLabel->setSizePolicy(sizePolicy1);
        iconLabel->setText(QString::fromUtf8("ICON"));

        horizontalLayout->addWidget(iconLabel);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        nameLabel = new QLabel(ExecutableFileOpenDialog);
        nameLabel->setObjectName("nameLabel");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(nameLabel->sizePolicy().hasHeightForWidth());
        nameLabel->setSizePolicy(sizePolicy2);
        nameLabel->setText(QString::fromUtf8("FILE NAME"));
        nameLabel->setTextFormat(Qt::TextFormat::PlainText);
        nameLabel->setTextInteractionFlags(Qt::TextInteractionFlag::LinksAccessibleByMouse|Qt::TextInteractionFlag::TextSelectableByMouse);

        verticalLayout_2->addWidget(nameLabel, 0, Qt::AlignmentFlag::AlignBottom);

        mimeTypeLabel = new QLabel(ExecutableFileOpenDialog);
        mimeTypeLabel->setObjectName("mimeTypeLabel");
        sizePolicy2.setHeightForWidth(mimeTypeLabel->sizePolicy().hasHeightForWidth());
        mimeTypeLabel->setSizePolicy(sizePolicy2);
        mimeTypeLabel->setText(QString::fromUtf8("FILE TYPE"));
        mimeTypeLabel->setTextFormat(Qt::TextFormat::PlainText);
        mimeTypeLabel->setTextInteractionFlags(Qt::TextInteractionFlag::LinksAccessibleByMouse|Qt::TextInteractionFlag::TextSelectableByMouse);

        verticalLayout_2->addWidget(mimeTypeLabel, 0, Qt::AlignmentFlag::AlignTop);


        horizontalLayout->addLayout(verticalLayout_2);


        verticalLayout->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(ExecutableFileOpenDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Orientation::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::StandardButton::Cancel);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(ExecutableFileOpenDialog);

        QMetaObject::connectSlotsByName(ExecutableFileOpenDialog);
    } // setupUi

    void retranslateUi(QDialog *ExecutableFileOpenDialog)
    {
        ExecutableFileOpenDialog->setWindowTitle(QString());
        label->setText(tr2i18n("Would you like to launch this file?", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ExecutableFileOpenDialog: public Ui_ExecutableFileOpenDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // EXECUTABLEFILEOPENDIALOG_H

