/****************************************************************************
** Meta object code from reading C++ file 'kdiroperator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.28.0/src/filewidgets/kdiroperator.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kdiroperator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12KDirOperatorE_t {};
} // unnamed namespace

template <> constexpr inline auto KDirOperator::qt_create_metaobjectdata<qt_meta_tag_ZN12KDirOperatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDirOperator",
        "urlEntered",
        "",
        "QUrl",
        "updateInformation",
        "files",
        "dirs",
        "completion",
        "finishedLoading",
        "viewChanged",
        "QAbstractItemView*",
        "newView",
        "fileHighlighted",
        "KFileItem",
        "item",
        "dirActivated",
        "fileSelected",
        "dropped",
        "QDropEvent*",
        "event",
        "QList<QUrl>",
        "urls",
        "contextMenuAboutToShow",
        "QMenu*",
        "menu",
        "currentIconSizeChanged",
        "size",
        "keyEnterReturnPressed",
        "renamingFinished",
        "back",
        "forward",
        "home",
        "cdUp",
        "updateDir",
        "rereadDir",
        "mkdir",
        "deleteSelected",
        "updateSelectionDependentActions",
        "makeCompletion",
        "makeDirCompletion",
        "renameSelected",
        "trashSelected",
        "setIconSize",
        "value",
        "setSupportedSchemes",
        "schemes",
        "resetCursor",
        "pathChanged",
        "selectDir",
        "selectFile",
        "highlightFile",
        "sortByName",
        "sortBySize",
        "sortByDate",
        "sortByType",
        "sortReversed",
        "toggleDirsFirst",
        "toggleIgnoreCase",
        "slotCompletionMatch",
        "match"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'urlEntered'
        QtMocHelpers::SignalData<void(const QUrl &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'updateInformation'
        QtMocHelpers::SignalData<void(int, int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'completion'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'finishedLoading'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewChanged'
        QtMocHelpers::SignalData<void(QAbstractItemView *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'fileHighlighted'
        QtMocHelpers::SignalData<void(const KFileItem &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'dirActivated'
        QtMocHelpers::SignalData<void(const KFileItem &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'fileSelected'
        QtMocHelpers::SignalData<void(const KFileItem &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'dropped'
        QtMocHelpers::SignalData<void(const KFileItem &, QDropEvent *, const QList<QUrl> &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 18, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Signal 'contextMenuAboutToShow'
        QtMocHelpers::SignalData<void(const KFileItem &, QMenu *)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 23, 24 },
        }}),
        // Signal 'currentIconSizeChanged'
        QtMocHelpers::SignalData<void(int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 },
        }}),
        // Signal 'keyEnterReturnPressed'
        QtMocHelpers::SignalData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'renamingFinished'
        QtMocHelpers::SignalData<void(const QList<QUrl> &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'back'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'forward'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'home'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cdUp'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateDir'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'rereadDir'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'mkdir'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deleteSelected'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateSelectionDependentActions'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'makeCompletion'
        QtMocHelpers::SlotData<QString(const QString &)>(38, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'makeDirCompletion'
        QtMocHelpers::SlotData<QString(const QString &)>(39, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'renameSelected'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trashSelected'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setIconSize'
        QtMocHelpers::SlotData<void(int)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 43 },
        }}),
        // Slot 'setSupportedSchemes'
        QtMocHelpers::SlotData<void(const QStringList &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 45 },
        }}),
        // Slot 'resetCursor'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'pathChanged'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'selectDir'
        QtMocHelpers::SlotData<void(const KFileItem &)>(48, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'selectFile'
        QtMocHelpers::SlotData<void(const KFileItem &)>(49, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'highlightFile'
        QtMocHelpers::SlotData<void(const KFileItem &)>(50, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'sortByName'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'sortBySize'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'sortByDate'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'sortByType'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'sortReversed'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'toggleDirsFirst'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'toggleIgnoreCase'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCompletionMatch'
        QtMocHelpers::SlotData<void(const QString &)>(58, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 59 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KDirOperator, qt_meta_tag_ZN12KDirOperatorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDirOperator::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KDirOperatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KDirOperatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12KDirOperatorE_t>.metaTypes,
    nullptr
} };

void KDirOperator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KDirOperator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->urlEntered((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 1: _t->updateInformation((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->completion((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->finishedLoading(); break;
        case 4: _t->viewChanged((*reinterpret_cast<std::add_pointer_t<QAbstractItemView*>>(_a[1]))); break;
        case 5: _t->fileHighlighted((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1]))); break;
        case 6: _t->dirActivated((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1]))); break;
        case 7: _t->fileSelected((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1]))); break;
        case 8: _t->dropped((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDropEvent*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[3]))); break;
        case 9: _t->contextMenuAboutToShow((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QMenu*>>(_a[2]))); break;
        case 10: _t->currentIconSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->keyEnterReturnPressed(); break;
        case 12: _t->renamingFinished((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 13: _t->back(); break;
        case 14: _t->forward(); break;
        case 15: _t->home(); break;
        case 16: _t->cdUp(); break;
        case 17: _t->updateDir(); break;
        case 18: _t->rereadDir(); break;
        case 19: _t->mkdir(); break;
        case 20: _t->deleteSelected(); break;
        case 21: _t->updateSelectionDependentActions(); break;
        case 22: { QString _r = _t->makeCompletion((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 23: { QString _r = _t->makeDirCompletion((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 24: _t->renameSelected(); break;
        case 25: _t->trashSelected(); break;
        case 26: _t->setIconSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 27: _t->setSupportedSchemes((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 28: _t->resetCursor(); break;
        case 29: _t->pathChanged(); break;
        case 30: _t->selectDir((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1]))); break;
        case 31: _t->selectFile((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1]))); break;
        case 32: _t->highlightFile((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1]))); break;
        case 33: _t->sortByName(); break;
        case 34: _t->sortBySize(); break;
        case 35: _t->sortByDate(); break;
        case 36: _t->sortByType(); break;
        case 37: _t->sortReversed(); break;
        case 38: _t->toggleDirsFirst(); break;
        case 39: _t->toggleIgnoreCase(); break;
        case 40: _t->slotCompletionMatch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KFileItem >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KFileItem >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KFileItem >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KFileItem >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KFileItem >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KFileItem >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KFileItem >(); break;
            }
            break;
        case 32:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KFileItem >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KDirOperator::*)(const QUrl & )>(_a, &KDirOperator::urlEntered, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KDirOperator::*)(int , int )>(_a, &KDirOperator::updateInformation, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KDirOperator::*)(const QString & )>(_a, &KDirOperator::completion, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KDirOperator::*)()>(_a, &KDirOperator::finishedLoading, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KDirOperator::*)(QAbstractItemView * )>(_a, &KDirOperator::viewChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KDirOperator::*)(const KFileItem & )>(_a, &KDirOperator::fileHighlighted, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KDirOperator::*)(const KFileItem & )>(_a, &KDirOperator::dirActivated, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KDirOperator::*)(const KFileItem & )>(_a, &KDirOperator::fileSelected, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KDirOperator::*)(const KFileItem & , QDropEvent * , const QList<QUrl> & )>(_a, &KDirOperator::dropped, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (KDirOperator::*)(const KFileItem & , QMenu * )>(_a, &KDirOperator::contextMenuAboutToShow, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (KDirOperator::*)(int )>(_a, &KDirOperator::currentIconSizeChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (KDirOperator::*)()>(_a, &KDirOperator::keyEnterReturnPressed, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (KDirOperator::*)(const QList<QUrl> & )>(_a, &KDirOperator::renamingFinished, 12))
            return;
    }
}

const QMetaObject *KDirOperator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDirOperator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KDirOperatorE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int KDirOperator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 41)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 41;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 41)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 41;
    }
    return _id;
}

// SIGNAL 0
void KDirOperator::urlEntered(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KDirOperator::updateInformation(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KDirOperator::completion(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KDirOperator::finishedLoading()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KDirOperator::viewChanged(QAbstractItemView * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KDirOperator::fileHighlighted(const KFileItem & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KDirOperator::dirActivated(const KFileItem & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void KDirOperator::fileSelected(const KFileItem & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void KDirOperator::dropped(const KFileItem & _t1, QDropEvent * _t2, const QList<QUrl> & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2, _t3);
}

// SIGNAL 9
void KDirOperator::contextMenuAboutToShow(const KFileItem & _t1, QMenu * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void KDirOperator::currentIconSizeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void KDirOperator::keyEnterReturnPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void KDirOperator::renamingFinished(const QList<QUrl> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}
QT_WARNING_POP
