/*
    This file is part of the KDE project
    SPDX-FileCopyrightText: 2009 Harald Hvaal <haraldhv@stud.ntnu.no>

    SPDX-License-Identifier: LGPL-2.0-only OR LGPL-3.0-only
*/

#include "dndpopupmenuplugin.h"

using namespace KIO;

DndPopupMenuPlugin::DndPopupMenuPlugin(QObject *parent)
    : QObject(parent)
{
}

DndPopupMenuPlugin::~DndPopupMenuPlugin()
{
}

#include "moc_dndpopupmenuplugin.cpp"
