/********************************************************************************
** Form generated from reading UI file 'knewfilemenu_newfiledialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KNEWFILEMENU_NEWFILEDIALOG_H
#define UI_KNEWFILEMENU_NEWFILEDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcollapsiblegroupbox.h"
#include "kmessagewidget.h"

QT_BEGIN_NAMESPACE

class Ui_NewFileDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QLabel *iconLabel;
    QLineEdit *lineEdit;
    KMessageWidget *messageWidget;
    KCollapsibleGroupBox *chooseIconBox;
    QVBoxLayout *verticalLayout_3;
    QWidget *widget;
    QVBoxLayout *verticalLayout_4;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *folderIconGrid;
    QPushButton *chooseIconButton;
    QLabel *iconHintLabel;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *NewFileDialog)
    {
        if (NewFileDialog->objectName().isEmpty())
            NewFileDialog->setObjectName("NewFileDialog");
        NewFileDialog->resize(453, 178);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(NewFileDialog->sizePolicy().hasHeightForWidth());
        NewFileDialog->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(NewFileDialog);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setSizeConstraint(QLayout::SizeConstraint::SetFixedSize);
        label = new QLabel(NewFileDialog);
        label->setObjectName("label");
        label->setTextFormat(Qt::TextFormat::PlainText);
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        iconLabel = new QLabel(NewFileDialog);
        iconLabel->setObjectName("iconLabel");
        iconLabel->setText(QString::fromUtf8("ICON"));

        horizontalLayout->addWidget(iconLabel);

        lineEdit = new QLineEdit(NewFileDialog);
        lineEdit->setObjectName("lineEdit");
        lineEdit->setMinimumSize(QSize(400, 0));
        lineEdit->setClearButtonEnabled(true);

        horizontalLayout->addWidget(lineEdit, 0, Qt::AlignmentFlag::AlignVCenter);


        verticalLayout->addLayout(horizontalLayout);

        messageWidget = new KMessageWidget(NewFileDialog);
        messageWidget->setObjectName("messageWidget");
        messageWidget->setWordWrap(true);
        messageWidget->setCloseButtonVisible(false);

        verticalLayout->addWidget(messageWidget);

        chooseIconBox = new KCollapsibleGroupBox(NewFileDialog);
        chooseIconBox->setObjectName("chooseIconBox");
        chooseIconBox->setExpanded(false);
        verticalLayout_3 = new QVBoxLayout(chooseIconBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(-1, -1, -1, 0);
        widget = new QWidget(chooseIconBox);
        widget->setObjectName("widget");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(widget->sizePolicy().hasHeightForWidth());
        widget->setSizePolicy(sizePolicy1);
        widget->setMinimumSize(QSize(0, 50));
        widget->setFocusPolicy(Qt::FocusPolicy::TabFocus);
        verticalLayout_4 = new QVBoxLayout(widget);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setSizeConstraint(QLayout::SizeConstraint::SetFixedSize);
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        folderIconGrid = new QGridLayout();
        folderIconGrid->setObjectName("folderIconGrid");

        verticalLayout_2->addLayout(folderIconGrid);

        chooseIconButton = new QPushButton(widget);
        chooseIconButton->setObjectName("chooseIconButton");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("dialog-icon-preview")));
        chooseIconButton->setIcon(icon);

        verticalLayout_2->addWidget(chooseIconButton);

        iconHintLabel = new QLabel(widget);
        iconHintLabel->setObjectName("iconHintLabel");
        sizePolicy1.setHeightForWidth(iconHintLabel->sizePolicy().hasHeightForWidth());
        iconHintLabel->setSizePolicy(sizePolicy1);
        iconHintLabel->setWordWrap(true);

        verticalLayout_2->addWidget(iconHintLabel);


        verticalLayout_4->addLayout(verticalLayout_2);


        verticalLayout_3->addWidget(widget);


        verticalLayout->addWidget(chooseIconBox);

        buttonBox = new QDialogButtonBox(NewFileDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::StandardButton::Cancel|QDialogButtonBox::StandardButton::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(lineEdit, chooseIconBox);
        QWidget::setTabOrder(chooseIconBox, widget);
        QWidget::setTabOrder(widget, chooseIconButton);

        retranslateUi(NewFileDialog);

        QMetaObject::connectSlotsByName(NewFileDialog);
    } // setupUi

    void retranslateUi(QDialog *NewFileDialog)
    {
        NewFileDialog->setWindowTitle(tr2i18n("Create New File", "@title:window"));
        label->setText(QString());
        lineEdit->setPlaceholderText(tr2i18n("Enter file name", nullptr));
        chooseIconBox->setTitle(tr2i18n("Choose custom icon or color", nullptr));
        chooseIconButton->setText(tr2i18n("Choose Other Icon\342\200\246", "@action:button"));
        iconHintLabel->setText(tr2i18n("You can change the icon later in folder properties.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class NewFileDialog: public Ui_NewFileDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KNEWFILEMENU_NEWFILEDIALOG_H

