/****************************************************************************
** Meta object code from reading C++ file 'renamedialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.28.0/src/widgets/renamedialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'renamedialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3KIO12RenameDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto KIO::RenameDialog::qt_create_metaobjectdata<qt_meta_tag_ZN3KIO12RenameDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KIO::RenameDialog",
        "cancelPressed",
        "",
        "renamePressed",
        "skipPressed",
        "overwritePressed",
        "overwriteAllPressed",
        "overwriteWhenOlderPressed",
        "resumePressed",
        "resumeAllPressed",
        "suggestNewNamePressed",
        "enableRenameButton",
        "applyAllPressed",
        "comparePressed",
        "showSrcIcon",
        "KFileItem",
        "showDestIcon",
        "showSrcPreview",
        "QPixmap",
        "showDestPreview",
        "resizePanels"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'cancelPressed'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'renamePressed'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'skipPressed'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'overwritePressed'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'overwriteAllPressed'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'overwriteWhenOlderPressed'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resumePressed'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resumeAllPressed'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'suggestNewNamePressed'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enableRenameButton'
        QtMocHelpers::SlotData<void(const QString &)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'applyAllPressed'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'comparePressed'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showSrcIcon'
        QtMocHelpers::SlotData<void(const KFileItem &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 2 },
        }}),
        // Slot 'showDestIcon'
        QtMocHelpers::SlotData<void(const KFileItem &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 2 },
        }}),
        // Slot 'showSrcPreview'
        QtMocHelpers::SlotData<void(const KFileItem &, const QPixmap &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 2 }, { 0x80000000 | 18, 2 },
        }}),
        // Slot 'showDestPreview'
        QtMocHelpers::SlotData<void(const KFileItem &, const QPixmap &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 2 }, { 0x80000000 | 18, 2 },
        }}),
        // Slot 'resizePanels'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RenameDialog, qt_meta_tag_ZN3KIO12RenameDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KIO::RenameDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO12RenameDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO12RenameDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3KIO12RenameDialogE_t>.metaTypes,
    nullptr
} };

void KIO::RenameDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RenameDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->cancelPressed(); break;
        case 1: _t->renamePressed(); break;
        case 2: _t->skipPressed(); break;
        case 3: _t->overwritePressed(); break;
        case 4: _t->overwriteAllPressed(); break;
        case 5: _t->overwriteWhenOlderPressed(); break;
        case 6: _t->resumePressed(); break;
        case 7: _t->resumeAllPressed(); break;
        case 8: _t->suggestNewNamePressed(); break;
        case 9: _t->enableRenameButton((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->applyAllPressed(); break;
        case 11: _t->comparePressed(); break;
        case 12: _t->showSrcIcon((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1]))); break;
        case 13: _t->showDestIcon((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1]))); break;
        case 14: _t->showSrcPreview((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[2]))); break;
        case 15: _t->showDestPreview((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[2]))); break;
        case 16: _t->resizePanels(); break;
        default: ;
        }
    }
}

const QMetaObject *KIO::RenameDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KIO::RenameDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO12RenameDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int KIO::RenameDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}
QT_WARNING_POP
