/****************************************************************************
** Meta object code from reading C++ file 'kpropertiesdialogbuiltin_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.28.0/src/widgets/kpropertiesdialogbuiltin_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kpropertiesdialogbuiltin_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KDEPrivate16KFilePropsPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto KDEPrivate::KFilePropsPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN10KDEPrivate16KFilePropsPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDEPrivate::KFilePropsPlugin",
        "changesApplied",
        "",
        "slotEditFileType",
        "slotCopyFinished",
        "KJob*",
        "slotFileRenamed",
        "KIO::Job*",
        "QUrl",
        "slotDirSizeUpdate",
        "slotDirSizeFinished",
        "slotFreeSpaceResult",
        "slotSizeStop",
        "slotSizeDetermine",
        "slotSizeDetails",
        "nameFileChanged",
        "text",
        "slotIconChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changesApplied'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditFileType'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCopyFinished'
        QtMocHelpers::SlotData<void(KJob *)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Slot 'slotFileRenamed'
        QtMocHelpers::SlotData<void(KIO::Job *, const QUrl &, const QUrl &)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 7, 2 }, { 0x80000000 | 8, 2 }, { 0x80000000 | 8, 2 },
        }}),
        // Slot 'slotDirSizeUpdate'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDirSizeFinished'
        QtMocHelpers::SlotData<void(KJob *)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Slot 'slotFreeSpaceResult'
        QtMocHelpers::SlotData<void(KJob *)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Slot 'slotSizeStop'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSizeDetermine'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSizeDetails'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'nameFileChanged'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Slot 'slotIconChanged'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KFilePropsPlugin, qt_meta_tag_ZN10KDEPrivate16KFilePropsPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDEPrivate::KFilePropsPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<KPropertiesDialogPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate16KFilePropsPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate16KFilePropsPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KDEPrivate16KFilePropsPluginE_t>.metaTypes,
    nullptr
} };

void KDEPrivate::KFilePropsPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KFilePropsPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changesApplied(); break;
        case 1: _t->slotEditFileType(); break;
        case 2: _t->slotCopyFinished((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 3: _t->slotFileRenamed((*reinterpret_cast<std::add_pointer_t<KIO::Job*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3]))); break;
        case 4: _t->slotDirSizeUpdate(); break;
        case 5: _t->slotDirSizeFinished((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 6: _t->slotFreeSpaceResult((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 7: _t->slotSizeStop(); break;
        case 8: _t->slotSizeDetermine(); break;
        case 9: _t->slotSizeDetails(); break;
        case 10: _t->nameFileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->slotIconChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KFilePropsPlugin::*)()>(_a, &KFilePropsPlugin::changesApplied, 0))
            return;
    }
}

const QMetaObject *KDEPrivate::KFilePropsPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDEPrivate::KFilePropsPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate16KFilePropsPluginE_t>.strings))
        return static_cast<void*>(this);
    return KPropertiesDialogPlugin::qt_metacast(_clname);
}

int KDEPrivate::KFilePropsPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPropertiesDialogPlugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void KDEPrivate::KFilePropsPlugin::changesApplied()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN10KDEPrivate27KFilePermissionsPropsPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto KDEPrivate::KFilePermissionsPropsPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN10KDEPrivate27KFilePermissionsPropsPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDEPrivate::KFilePermissionsPropsPlugin",
        "changesApplied",
        "",
        "slotShowAdvancedPermissions"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changesApplied'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowAdvancedPermissions'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KFilePermissionsPropsPlugin, qt_meta_tag_ZN10KDEPrivate27KFilePermissionsPropsPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDEPrivate::KFilePermissionsPropsPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<KPropertiesDialogPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate27KFilePermissionsPropsPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate27KFilePermissionsPropsPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KDEPrivate27KFilePermissionsPropsPluginE_t>.metaTypes,
    nullptr
} };

void KDEPrivate::KFilePermissionsPropsPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KFilePermissionsPropsPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changesApplied(); break;
        case 1: _t->slotShowAdvancedPermissions(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KFilePermissionsPropsPlugin::*)()>(_a, &KFilePermissionsPropsPlugin::changesApplied, 0))
            return;
    }
}

const QMetaObject *KDEPrivate::KFilePermissionsPropsPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDEPrivate::KFilePermissionsPropsPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate27KFilePermissionsPropsPluginE_t>.strings))
        return static_cast<void*>(this);
    return KPropertiesDialogPlugin::qt_metacast(_clname);
}

int KDEPrivate::KFilePermissionsPropsPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPropertiesDialogPlugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KDEPrivate::KFilePermissionsPropsPlugin::changesApplied()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN10KDEPrivate16KChecksumsPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto KDEPrivate::KChecksumsPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN10KDEPrivate16KChecksumsPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDEPrivate::KChecksumsPlugin",
        "slotInvalidateCache",
        "",
        "slotShowMd5",
        "slotShowSha1",
        "slotShowSha256",
        "slotShowSha512",
        "slotVerifyChecksum",
        "input"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotInvalidateCache'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowMd5'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowSha1'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowSha256'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowSha512'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotVerifyChecksum'
        QtMocHelpers::SlotData<void(const QString &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KChecksumsPlugin, qt_meta_tag_ZN10KDEPrivate16KChecksumsPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDEPrivate::KChecksumsPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<KPropertiesDialogPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate16KChecksumsPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate16KChecksumsPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KDEPrivate16KChecksumsPluginE_t>.metaTypes,
    nullptr
} };

void KDEPrivate::KChecksumsPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KChecksumsPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotInvalidateCache(); break;
        case 1: _t->slotShowMd5(); break;
        case 2: _t->slotShowSha1(); break;
        case 3: _t->slotShowSha256(); break;
        case 4: _t->slotShowSha512(); break;
        case 5: _t->slotVerifyChecksum((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KDEPrivate::KChecksumsPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDEPrivate::KChecksumsPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate16KChecksumsPluginE_t>.strings))
        return static_cast<void*>(this);
    return KPropertiesDialogPlugin::qt_metacast(_clname);
}

int KDEPrivate::KChecksumsPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPropertiesDialogPlugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN10KDEPrivate15KUrlPropsPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto KDEPrivate::KUrlPropsPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN10KDEPrivate15KUrlPropsPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDEPrivate::KUrlPropsPlugin"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KUrlPropsPlugin, qt_meta_tag_ZN10KDEPrivate15KUrlPropsPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDEPrivate::KUrlPropsPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<KPropertiesDialogPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate15KUrlPropsPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate15KUrlPropsPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KDEPrivate15KUrlPropsPluginE_t>.metaTypes,
    nullptr
} };

void KDEPrivate::KUrlPropsPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KUrlPropsPlugin *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KDEPrivate::KUrlPropsPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDEPrivate::KUrlPropsPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate15KUrlPropsPluginE_t>.strings))
        return static_cast<void*>(this);
    return KPropertiesDialogPlugin::qt_metacast(_clname);
}

int KDEPrivate::KUrlPropsPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPropertiesDialogPlugin::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN10KDEPrivate19KDesktopPropsPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto KDEPrivate::KDesktopPropsPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN10KDEPrivate19KDesktopPropsPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDEPrivate::KDesktopPropsPlugin",
        "slotAddFiletype",
        "",
        "slotDelFiletype",
        "slotBrowseExec",
        "slotAdvanced"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotAddFiletype'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDelFiletype'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotBrowseExec'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAdvanced'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KDesktopPropsPlugin, qt_meta_tag_ZN10KDEPrivate19KDesktopPropsPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDEPrivate::KDesktopPropsPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<KPropertiesDialogPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate19KDesktopPropsPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate19KDesktopPropsPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KDEPrivate19KDesktopPropsPluginE_t>.metaTypes,
    nullptr
} };

void KDEPrivate::KDesktopPropsPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KDesktopPropsPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotAddFiletype(); break;
        case 1: _t->slotDelFiletype(); break;
        case 2: _t->slotBrowseExec(); break;
        case 3: _t->slotAdvanced(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *KDEPrivate::KDesktopPropsPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDEPrivate::KDesktopPropsPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KDEPrivate19KDesktopPropsPluginE_t>.strings))
        return static_cast<void*>(this);
    return KPropertiesDialogPlugin::qt_metacast(_clname);
}

int KDEPrivate::KDesktopPropsPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPropertiesDialogPlugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
QT_WARNING_POP
