/****************************************************************************
** Meta object code from reading C++ file 'workerinterface_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.28.0/src/core/workerinterface_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'workerinterface_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3KIO15WorkerInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto KIO::WorkerInterface::qt_create_metaobjectdata<qt_meta_tag_ZN3KIO15WorkerInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KIO::WorkerInterface",
        "data",
        "",
        "dataReq",
        "error",
        "connected",
        "finished",
        "listEntries",
        "KIO::UDSEntryList",
        "statEntry",
        "KIO::UDSEntry",
        "canResume",
        "KIO::filesize_t",
        "open",
        "written",
        "close",
        "privilegeOperationRequested",
        "metaData",
        "KIO::MetaData",
        "totalSize",
        "processedSize",
        "redirection",
        "QUrl",
        "position",
        "truncated",
        "speed",
        "mimeType",
        "warning",
        "infoMessage",
        "calcSpeed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'data'
        QtMocHelpers::SignalData<void(const QByteArray &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 2 },
        }}),
        // Signal 'dataReq'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'error'
        QtMocHelpers::SignalData<void(int, const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'connected'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'listEntries'
        QtMocHelpers::SignalData<void(const KIO::UDSEntryList &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Signal 'statEntry'
        QtMocHelpers::SignalData<void(const KIO::UDSEntry &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Signal 'canResume'
        QtMocHelpers::SignalData<void(KIO::filesize_t)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Signal 'open'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'written'
        QtMocHelpers::SignalData<void(KIO::filesize_t)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Signal 'close'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'privilegeOperationRequested'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'metaData'
        QtMocHelpers::SignalData<void(const KIO::MetaData &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 2 },
        }}),
        // Signal 'totalSize'
        QtMocHelpers::SignalData<void(KIO::filesize_t)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Signal 'processedSize'
        QtMocHelpers::SignalData<void(KIO::filesize_t)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Signal 'redirection'
        QtMocHelpers::SignalData<void(const QUrl &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 2 },
        }}),
        // Signal 'position'
        QtMocHelpers::SignalData<void(KIO::filesize_t)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Signal 'truncated'
        QtMocHelpers::SignalData<void(KIO::filesize_t)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Signal 'speed'
        QtMocHelpers::SignalData<void(unsigned long)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULong, 2 },
        }}),
        // Signal 'mimeType'
        QtMocHelpers::SignalData<void(const QString &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'warning'
        QtMocHelpers::SignalData<void(const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'infoMessage'
        QtMocHelpers::SignalData<void(const QString &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'calcSpeed'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WorkerInterface, qt_meta_tag_ZN3KIO15WorkerInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KIO::WorkerInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO15WorkerInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO15WorkerInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3KIO15WorkerInterfaceE_t>.metaTypes,
    nullptr
} };

void KIO::WorkerInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WorkerInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->data((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 1: _t->dataReq(); break;
        case 2: _t->error((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->connected(); break;
        case 4: _t->finished(); break;
        case 5: _t->listEntries((*reinterpret_cast<std::add_pointer_t<KIO::UDSEntryList>>(_a[1]))); break;
        case 6: _t->statEntry((*reinterpret_cast<std::add_pointer_t<KIO::UDSEntry>>(_a[1]))); break;
        case 7: _t->canResume((*reinterpret_cast<std::add_pointer_t<KIO::filesize_t>>(_a[1]))); break;
        case 8: _t->open(); break;
        case 9: _t->written((*reinterpret_cast<std::add_pointer_t<KIO::filesize_t>>(_a[1]))); break;
        case 10: _t->close(); break;
        case 11: _t->privilegeOperationRequested(); break;
        case 12: _t->metaData((*reinterpret_cast<std::add_pointer_t<KIO::MetaData>>(_a[1]))); break;
        case 13: _t->totalSize((*reinterpret_cast<std::add_pointer_t<KIO::filesize_t>>(_a[1]))); break;
        case 14: _t->processedSize((*reinterpret_cast<std::add_pointer_t<KIO::filesize_t>>(_a[1]))); break;
        case 15: _t->redirection((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 16: _t->position((*reinterpret_cast<std::add_pointer_t<KIO::filesize_t>>(_a[1]))); break;
        case 17: _t->truncated((*reinterpret_cast<std::add_pointer_t<KIO::filesize_t>>(_a[1]))); break;
        case 18: _t->speed((*reinterpret_cast<std::add_pointer_t<ulong>>(_a[1]))); break;
        case 19: _t->mimeType((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->warning((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->infoMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->calcSpeed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KIO::UDSEntry >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(const QByteArray & )>(_a, &WorkerInterface::data, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)()>(_a, &WorkerInterface::dataReq, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(int , const QString & )>(_a, &WorkerInterface::error, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)()>(_a, &WorkerInterface::connected, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)()>(_a, &WorkerInterface::finished, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(const KIO::UDSEntryList & )>(_a, &WorkerInterface::listEntries, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(const KIO::UDSEntry & )>(_a, &WorkerInterface::statEntry, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(KIO::filesize_t )>(_a, &WorkerInterface::canResume, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)()>(_a, &WorkerInterface::open, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(KIO::filesize_t )>(_a, &WorkerInterface::written, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)()>(_a, &WorkerInterface::close, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)()>(_a, &WorkerInterface::privilegeOperationRequested, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(const KIO::MetaData & )>(_a, &WorkerInterface::metaData, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(KIO::filesize_t )>(_a, &WorkerInterface::totalSize, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(KIO::filesize_t )>(_a, &WorkerInterface::processedSize, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(const QUrl & )>(_a, &WorkerInterface::redirection, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(KIO::filesize_t )>(_a, &WorkerInterface::position, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(KIO::filesize_t )>(_a, &WorkerInterface::truncated, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(unsigned long )>(_a, &WorkerInterface::speed, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(const QString & )>(_a, &WorkerInterface::mimeType, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(const QString & )>(_a, &WorkerInterface::warning, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerInterface::*)(const QString & )>(_a, &WorkerInterface::infoMessage, 21))
            return;
    }
}

const QMetaObject *KIO::WorkerInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KIO::WorkerInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO15WorkerInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KIO::WorkerInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void KIO::WorkerInterface::data(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KIO::WorkerInterface::dataReq()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KIO::WorkerInterface::error(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void KIO::WorkerInterface::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KIO::WorkerInterface::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KIO::WorkerInterface::listEntries(const KIO::UDSEntryList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KIO::WorkerInterface::statEntry(const KIO::UDSEntry & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void KIO::WorkerInterface::canResume(KIO::filesize_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void KIO::WorkerInterface::open()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void KIO::WorkerInterface::written(KIO::filesize_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void KIO::WorkerInterface::close()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void KIO::WorkerInterface::privilegeOperationRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void KIO::WorkerInterface::metaData(const KIO::MetaData & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void KIO::WorkerInterface::totalSize(KIO::filesize_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void KIO::WorkerInterface::processedSize(KIO::filesize_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void KIO::WorkerInterface::redirection(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void KIO::WorkerInterface::position(KIO::filesize_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void KIO::WorkerInterface::truncated(KIO::filesize_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void KIO::WorkerInterface::speed(unsigned long _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void KIO::WorkerInterface::mimeType(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void KIO::WorkerInterface::warning(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void KIO::WorkerInterface::infoMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}
QT_WARNING_POP
