/****************************************************************************
** Meta object code from reading C++ file 'kfileitemdelegate.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.28.0/src/widgets/kfileitemdelegate.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kfileitemdelegate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17KFileItemDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto KFileItemDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN17KFileItemDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KFileItemDelegate",
        "helpEvent",
        "",
        "QHelpEvent*",
        "event",
        "QAbstractItemView*",
        "view",
        "QStyleOptionViewItem",
        "option",
        "QModelIndex",
        "index",
        "shape",
        "QRegion",
        "information",
        "InformationList",
        "shadowColor",
        "QColor",
        "shadowOffset",
        "QPointF",
        "shadowBlur",
        "maximumSize",
        "QSize",
        "showToolTipWhenElided",
        "jobTransfersVisible",
        "Information",
        "NoInformation",
        "Size",
        "Permissions",
        "OctalPermissions",
        "Owner",
        "OwnerAndGroup",
        "CreationTime",
        "ModificationTime",
        "AccessTime",
        "MimeType",
        "FriendlyMimeType",
        "LinkDest",
        "LocalPathOrUrl",
        "Comment"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'helpEvent'
        QtMocHelpers::SlotData<bool(QHelpEvent *, QAbstractItemView *, const QStyleOptionViewItem &, const QModelIndex &)>(1, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'shape'
        QtMocHelpers::SlotData<QRegion(const QStyleOptionViewItem &, const QModelIndex &)>(11, 2, QMC::AccessPublic, 0x80000000 | 12, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'information'
        QtMocHelpers::PropertyData<InformationList>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'shadowColor'
        QtMocHelpers::PropertyData<QColor>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'shadowOffset'
        QtMocHelpers::PropertyData<QPointF>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'shadowBlur'
        QtMocHelpers::PropertyData<qreal>(19, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'maximumSize'
        QtMocHelpers::PropertyData<QSize>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'showToolTipWhenElided'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'jobTransfersVisible'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Information'
        QtMocHelpers::EnumData<enum Information>(24, 24, QMC::EnumFlags{}).add({
            {   25, Information::NoInformation },
            {   26, Information::Size },
            {   27, Information::Permissions },
            {   28, Information::OctalPermissions },
            {   29, Information::Owner },
            {   30, Information::OwnerAndGroup },
            {   31, Information::CreationTime },
            {   32, Information::ModificationTime },
            {   33, Information::AccessTime },
            {   34, Information::MimeType },
            {   35, Information::FriendlyMimeType },
            {   36, Information::LinkDest },
            {   37, Information::LocalPathOrUrl },
            {   38, Information::Comment },
        }),
    };
    return QtMocHelpers::metaObjectData<KFileItemDelegate, qt_meta_tag_ZN17KFileItemDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KFileItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemDelegate::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KFileItemDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KFileItemDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17KFileItemDelegateE_t>.metaTypes,
    nullptr
} };

void KFileItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KFileItemDelegate *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { bool _r = _t->helpEvent((*reinterpret_cast<std::add_pointer_t<QHelpEvent*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAbstractItemView*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStyleOptionViewItem>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { QRegion _r = _t->shape((*reinterpret_cast<std::add_pointer_t<QStyleOptionViewItem>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QRegion*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<InformationList*>(_v) = _t->showInformation(); break;
        case 1: *reinterpret_cast<QColor*>(_v) = _t->shadowColor(); break;
        case 2: *reinterpret_cast<QPointF*>(_v) = _t->shadowOffset(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->shadowBlur(); break;
        case 4: *reinterpret_cast<QSize*>(_v) = _t->maximumSize(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->showToolTipWhenElided(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->jobTransfersVisible(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setShowInformation(*reinterpret_cast<InformationList*>(_v)); break;
        case 1: _t->setShadowColor(*reinterpret_cast<QColor*>(_v)); break;
        case 2: _t->setShadowOffset(*reinterpret_cast<QPointF*>(_v)); break;
        case 3: _t->setShadowBlur(*reinterpret_cast<qreal*>(_v)); break;
        case 4: _t->setMaximumSize(*reinterpret_cast<QSize*>(_v)); break;
        case 5: _t->setShowToolTipWhenElided(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setJobTransfersVisible(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KFileItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KFileItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KFileItemDelegateE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemDelegate::qt_metacast(_clname);
}

int KFileItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemDelegate::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
