/****************************************************************************
** Meta object code from reading C++ file 'fileundomanager_adaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "fileundomanager_adaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileundomanager_adaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN25KIOFileUndoManagerAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto KIOFileUndoManagerAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN25KIOFileUndoManagerAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KIOFileUndoManagerAdaptor",
        "D-Bus Interface",
        "org.kde.libkonq.FileUndoManager",
        "D-Bus Introspection",
        "  <interface name=\"org.kde.libkonq.FileUndoManager\">\n    <metho"
        "d name=\"get\">\n      <arg direction=\"out\" type=\"ay\" name=\"c"
        "ommands\"/>\n    </method>\n    <signal name=\"lock\"/>\n    <sign"
        "al name=\"pop\"/>\n    <signal name=\"push\">\n      <arg directio"
        "n=\"out\" type=\"ay\" name=\"command\"/>\n    </signal>\n    <sign"
        "al name=\"unlock\"/>\n  </interface>\n",
        "lock",
        "",
        "pop",
        "push",
        "command",
        "unlock",
        "get"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'lock'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pop'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'push'
        QtMocHelpers::SignalData<void(const QByteArray &)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 9 },
        }}),
        // Signal 'unlock'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'get'
        QtMocHelpers::SlotData<QByteArray()>(11, 6, QMC::AccessPublic, QMetaType::QByteArray),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<KIOFileUndoManagerAdaptor, qt_meta_tag_ZN25KIOFileUndoManagerAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject KIOFileUndoManagerAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25KIOFileUndoManagerAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25KIOFileUndoManagerAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25KIOFileUndoManagerAdaptorE_t>.metaTypes,
    nullptr
} };

void KIOFileUndoManagerAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KIOFileUndoManagerAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->lock(); break;
        case 1: _t->pop(); break;
        case 2: _t->push((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 3: _t->unlock(); break;
        case 4: { QByteArray _r = _t->get();
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KIOFileUndoManagerAdaptor::*)()>(_a, &KIOFileUndoManagerAdaptor::lock, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KIOFileUndoManagerAdaptor::*)()>(_a, &KIOFileUndoManagerAdaptor::pop, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KIOFileUndoManagerAdaptor::*)(const QByteArray & )>(_a, &KIOFileUndoManagerAdaptor::push, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KIOFileUndoManagerAdaptor::*)()>(_a, &KIOFileUndoManagerAdaptor::unlock, 3))
            return;
    }
}

const QMetaObject *KIOFileUndoManagerAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KIOFileUndoManagerAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25KIOFileUndoManagerAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int KIOFileUndoManagerAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void KIOFileUndoManagerAdaptor::lock()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KIOFileUndoManagerAdaptor::pop()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KIOFileUndoManagerAdaptor::push(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KIOFileUndoManagerAdaptor::unlock()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
