/****************************************************************************
** Meta object code from reading C++ file 'kdirmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.28.0/src/widgets/kdirmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kdirmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9KDirModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KDirModel::qt_create_metaobjectdata<qt_meta_tag_ZN9KDirModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDirModel",
        "expand",
        "",
        "QModelIndex",
        "index",
        "needSequenceIcon",
        "sequenceIndex",
        "openUrl",
        "QUrl",
        "url",
        "OpenUrlFlags",
        "flags",
        "indexForItem",
        "KFileItem",
        "indexForUrl",
        "expandToUrl",
        "itemChanged",
        "clearAllPreviews",
        "setDropsAllowed",
        "DropsAllowed",
        "dropsAllowed",
        "OpenUrlFlag",
        "NoFlags",
        "Reload",
        "ShowRoot",
        "DropsAllowedFlag",
        "NoDrops",
        "DropOnDirectory",
        "DropOnAnyFile",
        "DropOnLocalExecutable"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'expand'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'needSequenceIcon'
        QtMocHelpers::SignalData<void(const QModelIndex &, int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 6 },
        }}),
        // Method 'openUrl'
        QtMocHelpers::MethodData<void(const QUrl &, OpenUrlFlags)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Method 'openUrl'
        QtMocHelpers::MethodData<void(const QUrl &)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Method 'indexForItem'
        QtMocHelpers::MethodData<QModelIndex(const KFileItem &) const>(12, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { 0x80000000 | 13, 2 },
        }}),
        // Method 'indexForUrl'
        QtMocHelpers::MethodData<QModelIndex(const QUrl &) const>(14, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Method 'expandToUrl'
        QtMocHelpers::MethodData<void(const QUrl &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Method 'itemChanged'
        QtMocHelpers::MethodData<void(const QModelIndex &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Method 'clearAllPreviews'
        QtMocHelpers::MethodData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'setDropsAllowed'
        QtMocHelpers::MethodData<void(DropsAllowed)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'OpenUrlFlags'
        QtMocHelpers::EnumData<OpenUrlFlags>(10, 21, QMC::EnumIsFlag).add({
            {   22, OpenUrlFlag::NoFlags },
            {   23, OpenUrlFlag::Reload },
            {   24, OpenUrlFlag::ShowRoot },
        }),
        // flag 'DropsAllowed'
        QtMocHelpers::EnumData<DropsAllowed>(19, 25, QMC::EnumIsFlag).add({
            {   26, DropsAllowedFlag::NoDrops },
            {   27, DropsAllowedFlag::DropOnDirectory },
            {   28, DropsAllowedFlag::DropOnAnyFile },
            {   29, DropsAllowedFlag::DropOnLocalExecutable },
        }),
    };
    return QtMocHelpers::metaObjectData<KDirModel, qt_meta_tag_ZN9KDirModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KDirModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KDirModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KDirModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9KDirModelE_t>.metaTypes,
    nullptr
} };

void KDirModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KDirModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->expand((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 1: _t->needSequenceIcon((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->openUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<OpenUrlFlags>>(_a[2]))); break;
        case 3: _t->openUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 4: { QModelIndex _r = _t->indexForItem((*reinterpret_cast<std::add_pointer_t<KFileItem>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 5: { QModelIndex _r = _t->indexForUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->expandToUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 7: _t->itemChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 8: _t->clearAllPreviews(); break;
        case 9: _t->setDropsAllowed((*reinterpret_cast<std::add_pointer_t<DropsAllowed>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KFileItem >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KDirModel::*)(const QModelIndex & )>(_a, &KDirModel::expand, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KDirModel::*)(const QModelIndex & , int )>(_a, &KDirModel::needSequenceIcon, 1))
            return;
    }
}

const QMetaObject *KDirModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDirModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KDirModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int KDirModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void KDirModel::expand(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KDirModel::needSequenceIcon(const QModelIndex & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
