/****************************************************************************
** Meta object code from reading C++ file 'dropjob.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.28.0/src/widgets/dropjob.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dropjob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3KIO7DropJobE_t {};
} // unnamed namespace

template <> constexpr inline auto KIO::DropJob::qt_create_metaobjectdata<qt_meta_tag_ZN3KIO7DropJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KIO::DropJob",
        "itemCreated",
        "",
        "QUrl",
        "url",
        "copyJobStarted",
        "KIO::CopyJob*",
        "job",
        "popupMenuAboutToShow",
        "KFileItemListProperties",
        "itemProps",
        "slotResult",
        "KJob*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'itemCreated'
        QtMocHelpers::SignalData<void(const QUrl &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'copyJobStarted'
        QtMocHelpers::SignalData<void(KIO::CopyJob *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'popupMenuAboutToShow'
        QtMocHelpers::SignalData<void(const KFileItemListProperties &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotResult'
        QtMocHelpers::SlotData<void(KJob *)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DropJob, qt_meta_tag_ZN3KIO7DropJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KIO::DropJob::staticMetaObject = { {
    QMetaObject::SuperData::link<Job::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO7DropJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO7DropJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3KIO7DropJobE_t>.metaTypes,
    nullptr
} };

void KIO::DropJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DropJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemCreated((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 1: _t->copyJobStarted((*reinterpret_cast<std::add_pointer_t<KIO::CopyJob*>>(_a[1]))); break;
        case 2: _t->popupMenuAboutToShow((*reinterpret_cast<std::add_pointer_t<KFileItemListProperties>>(_a[1]))); break;
        case 3: _t->slotResult((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DropJob::*)(const QUrl & )>(_a, &DropJob::itemCreated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DropJob::*)(KIO::CopyJob * )>(_a, &DropJob::copyJobStarted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DropJob::*)(const KFileItemListProperties & )>(_a, &DropJob::popupMenuAboutToShow, 2))
            return;
    }
}

const QMetaObject *KIO::DropJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KIO::DropJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KIO7DropJobE_t>.strings))
        return static_cast<void*>(this);
    return Job::qt_metacast(_clname);
}

int KIO::DropJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Job::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void KIO::DropJob::itemCreated(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KIO::DropJob::copyJobStarted(KIO::CopyJob * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KIO::DropJob::popupMenuAboutToShow(const KFileItemListProperties & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN3KIOE_t {};
} // unnamed namespace

template <> constexpr inline auto KIO::qt_create_metaobjectdata<qt_meta_tag_ZN3KIOE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KIO",
        "DndBehavior",
        "AlwaysAsk",
        "MoveIfSameDevice"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DndBehavior'
        QtMocHelpers::EnumData<DndBehavior>(1, 1, QMC::EnumFlags{}).add({
            {    2, DndBehavior::AlwaysAsk },
            {    3, DndBehavior::MoveIfSameDevice },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN3KIOE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN3KIOE =
    KIO::qt_create_metaobjectdata<qt_meta_tag_ZN3KIOE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN3KIOE =
    qt_staticMetaObjectContent_ZN3KIOE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN3KIOE =
    qt_staticMetaObjectContent_ZN3KIOE.relocatingData;

Q_CONSTINIT const QMetaObject KIO::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN3KIOE.stringdata,
    qt_staticMetaObjectStaticContent_ZN3KIOE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN3KIOE.metaTypes,
    nullptr
} };

QT_WARNING_POP
