/****************************************************************************
** Meta object code from reading C++ file 'kpasswdserver.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.28.0/src/kpasswdserver/kpasswdserver.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kpasswdserver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KPasswdServerE_t {};
} // unnamed namespace

template <> constexpr inline auto KPasswdServer::qt_create_metaobjectdata<qt_meta_tag_ZN13KPasswdServerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPasswdServer",
        "checkAuthInfoAsyncResult",
        "",
        "requestId",
        "seqNr",
        "KIO::AuthInfo",
        "queryAuthInfoAsyncResult",
        "checkAuthInfoAsync",
        "queryAuthInfoAsync",
        "addAuthInfo",
        "removeAuthInfo",
        "host",
        "protocol",
        "user",
        "checkAuthInfo",
        "queryAuthInfo",
        "processRequest",
        "removeAuthForWindowId",
        "windowId",
        "passwordDialogDone",
        "result",
        "KPasswordDialog*",
        "sender",
        "retryDialogDone",
        "KMessageDialog*",
        "windowRemoved",
        "WId"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'checkAuthInfoAsyncResult'
        QtMocHelpers::SignalData<void(qlonglong, qlonglong, const KIO::AuthInfo &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 3 }, { QMetaType::LongLong, 4 }, { 0x80000000 | 5, 2 },
        }}),
        // Signal 'queryAuthInfoAsyncResult'
        QtMocHelpers::SignalData<void(qlonglong, qlonglong, const KIO::AuthInfo &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 3 }, { QMetaType::LongLong, 4 }, { 0x80000000 | 5, 2 },
        }}),
        // Slot 'checkAuthInfoAsync'
        QtMocHelpers::SlotData<qlonglong(KIO::AuthInfo, qlonglong, qlonglong)>(7, 2, QMC::AccessPublic, QMetaType::LongLong, {{
            { 0x80000000 | 5, 2 }, { QMetaType::LongLong, 2 }, { QMetaType::LongLong, 2 },
        }}),
        // Slot 'queryAuthInfoAsync'
        QtMocHelpers::SlotData<qlonglong(const KIO::AuthInfo &, const QString &, qlonglong, qlonglong, qlonglong)>(8, 2, QMC::AccessPublic, QMetaType::LongLong, {{
            { 0x80000000 | 5, 2 }, { QMetaType::QString, 2 }, { QMetaType::LongLong, 2 }, { QMetaType::LongLong, 2 },
            { QMetaType::LongLong, 2 },
        }}),
        // Slot 'addAuthInfo'
        QtMocHelpers::SlotData<void(const KIO::AuthInfo &, qlonglong)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 }, { QMetaType::LongLong, 2 },
        }}),
        // Slot 'removeAuthInfo'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::QString, 13 },
        }}),
        // Slot 'checkAuthInfo'
        QtMocHelpers::SlotData<QByteArray(const QByteArray &, qlonglong, qlonglong)>(14, 2, QMC::AccessPublic, QMetaType::QByteArray, {{
            { QMetaType::QByteArray, 2 }, { QMetaType::LongLong, 2 }, { QMetaType::LongLong, 2 },
        }}),
        // Slot 'queryAuthInfo'
        QtMocHelpers::SlotData<QByteArray(const QByteArray &, const QString &, qlonglong, qlonglong, qlonglong)>(15, 2, QMC::AccessPublic, QMetaType::QByteArray, {{
            { QMetaType::QByteArray, 2 }, { QMetaType::QString, 2 }, { QMetaType::LongLong, 2 }, { QMetaType::LongLong, 2 },
            { QMetaType::LongLong, 2 },
        }}),
        // Slot 'addAuthInfo'
        QtMocHelpers::SlotData<void(const QByteArray &, qlonglong)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 2 }, { QMetaType::LongLong, 2 },
        }}),
        // Slot 'processRequest'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'removeAuthForWindowId'
        QtMocHelpers::SlotData<void(qlonglong)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 18 },
        }}),
        // Slot 'passwordDialogDone'
        QtMocHelpers::SlotData<void(int, KPasswordDialog *)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 20 }, { 0x80000000 | 21, 22 },
        }}),
        // Slot 'retryDialogDone'
        QtMocHelpers::SlotData<void(int, KMessageDialog *)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 20 }, { 0x80000000 | 24, 22 },
        }}),
        // Slot 'windowRemoved'
        QtMocHelpers::SlotData<void(WId)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KPasswdServer, qt_meta_tag_ZN13KPasswdServerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPasswdServer::staticMetaObject = { {
    QMetaObject::SuperData::link<KDEDModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KPasswdServerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KPasswdServerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KPasswdServerE_t>.metaTypes,
    nullptr
} };

void KPasswdServer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KPasswdServer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->checkAuthInfoAsyncResult((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KIO::AuthInfo>>(_a[3]))); break;
        case 1: _t->queryAuthInfoAsyncResult((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KIO::AuthInfo>>(_a[3]))); break;
        case 2: { qlonglong _r = _t->checkAuthInfoAsync((*reinterpret_cast<std::add_pointer_t<KIO::AuthInfo>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[3])));
            if (_a[0]) *reinterpret_cast<qlonglong*>(_a[0]) = std::move(_r); }  break;
        case 3: { qlonglong _r = _t->queryAuthInfoAsync((*reinterpret_cast<std::add_pointer_t<KIO::AuthInfo>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[5])));
            if (_a[0]) *reinterpret_cast<qlonglong*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->addAuthInfo((*reinterpret_cast<std::add_pointer_t<KIO::AuthInfo>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2]))); break;
        case 5: _t->removeAuthInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 6: { QByteArray _r = _t->checkAuthInfo((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 7: { QByteArray _r = _t->queryAuthInfo((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->addAuthInfo((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2]))); break;
        case 9: _t->processRequest(); break;
        case 10: _t->removeAuthForWindowId((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1]))); break;
        case 11: _t->passwordDialogDone((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPasswordDialog*>>(_a[2]))); break;
        case 12: _t->retryDialogDone((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KMessageDialog*>>(_a[2]))); break;
        case 13: _t->windowRemoved((*reinterpret_cast<std::add_pointer_t<WId>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KIO::AuthInfo >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KIO::AuthInfo >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KIO::AuthInfo >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KIO::AuthInfo >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KIO::AuthInfo >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KPasswdServer::*)(qlonglong , qlonglong , const KIO::AuthInfo & )>(_a, &KPasswdServer::checkAuthInfoAsyncResult, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KPasswdServer::*)(qlonglong , qlonglong , const KIO::AuthInfo & )>(_a, &KPasswdServer::queryAuthInfoAsyncResult, 1))
            return;
    }
}

const QMetaObject *KPasswdServer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPasswdServer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KPasswdServerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return KDEDModule::qt_metacast(_clname);
}

int KPasswdServer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KDEDModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void KPasswdServer::checkAuthInfoAsyncResult(qlonglong _t1, qlonglong _t2, const KIO::AuthInfo & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void KPasswdServer::queryAuthInfoAsyncResult(qlonglong _t1, qlonglong _t2, const KIO::AuthInfo & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
