/********************************************************************************
** Form generated from reading UI file 'sslinfo.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SSLINFO_H
#define UI_SSLINFO_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "ksqueezedtextlabel.h"

QT_BEGIN_NAMESPACE

class Ui_SslInfo
{
public:
    QGridLayout *gridLayout;
    QLabel *encryptionIndicator;
    QLabel *explanation;
    QLabel *addressTag;
    KSqueezedTextLabel *address;
    QLabel *ipTag;
    QLabel *ip;
    QLabel *encryptionTag;
    QLabel *encryption;
    QLabel *detailsTag;
    QLabel *details;
    QLabel *sslVersionTag;
    QLabel *sslVersion;
    QLabel *certSelectorTag;
    QComboBox *certSelector;
    QSpacerItem *spacerItem;
    QTabWidget *certParties;
    QLabel *trustedTag;
    QLabel *trusted;
    QLabel *validityPeriodTag;
    QLabel *validityPeriod;
    QLabel *serialTag;
    QLabel *serial;
    QLabel *digestTag;
    QLabel *digest;
    QLabel *sha1DigestTag;
    QLabel *sha1Digest;

    void setupUi(QWidget *SslInfo)
    {
        if (SslInfo->objectName().isEmpty())
            SslInfo->setObjectName("SslInfo");
        SslInfo->resize(474, 510);
        gridLayout = new QGridLayout(SslInfo);
        gridLayout->setObjectName("gridLayout");
        encryptionIndicator = new QLabel(SslInfo);
        encryptionIndicator->setObjectName("encryptionIndicator");

        gridLayout->addWidget(encryptionIndicator, 0, 0, 1, 1);

        explanation = new QLabel(SslInfo);
        explanation->setObjectName("explanation");
        explanation->setText(QString::fromUtf8("Bruce Schneier secure"));
        explanation->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayout->addWidget(explanation, 0, 1, 1, 2);

        addressTag = new QLabel(SslInfo);
        addressTag->setObjectName("addressTag");
        addressTag->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(addressTag, 1, 0, 1, 1);

        address = new KSqueezedTextLabel(SslInfo);
        address->setObjectName("address");
        address->setText(QString::fromUtf8("KSqueezedTextLabel"));
        address->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayout->addWidget(address, 1, 1, 1, 1);

        ipTag = new QLabel(SslInfo);
        ipTag->setObjectName("ipTag");
        ipTag->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(ipTag, 2, 0, 1, 1);

        ip = new QLabel(SslInfo);
        ip->setObjectName("ip");
        ip->setText(QString::fromUtf8("is not there"));
        ip->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayout->addWidget(ip, 2, 1, 1, 1);

        encryptionTag = new QLabel(SslInfo);
        encryptionTag->setObjectName("encryptionTag");
        encryptionTag->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(encryptionTag, 3, 0, 1, 1);

        encryption = new QLabel(SslInfo);
        encryption->setObjectName("encryption");
        encryption->setText(QString::fromUtf8("SnakeOilCrypt 3000"));
        encryption->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayout->addWidget(encryption, 3, 1, 1, 2);

        detailsTag = new QLabel(SslInfo);
        detailsTag->setObjectName("detailsTag");
        detailsTag->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(detailsTag, 4, 0, 1, 1);

        details = new QLabel(SslInfo);
        details->setObjectName("details");
        details->setText(QString::fromUtf8("Kx = DH, Auth = RSA, MAC = SHA1"));
        details->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayout->addWidget(details, 4, 1, 1, 2);

        sslVersionTag = new QLabel(SslInfo);
        sslVersionTag->setObjectName("sslVersionTag");
        sslVersionTag->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(sslVersionTag, 5, 0, 1, 1);

        sslVersion = new QLabel(SslInfo);
        sslVersion->setObjectName("sslVersion");
        sslVersion->setText(QString::fromUtf8("ElboniaTLS v0.0.0"));
        sslVersion->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayout->addWidget(sslVersion, 5, 1, 1, 1);

        certSelectorTag = new QLabel(SslInfo);
        certSelectorTag->setObjectName("certSelectorTag");
        certSelectorTag->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(certSelectorTag, 6, 0, 1, 1);

        certSelector = new QComboBox(SslInfo);
        certSelector->setObjectName("certSelector");

        gridLayout->addWidget(certSelector, 6, 1, 1, 1);

        spacerItem = new QSpacerItem(239, 16, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem, 6, 2, 1, 1);

        certParties = new QTabWidget(SslInfo);
        certParties->setObjectName("certParties");

        gridLayout->addWidget(certParties, 7, 0, 1, 3);

        trustedTag = new QLabel(SslInfo);
        trustedTag->setObjectName("trustedTag");
        trustedTag->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(trustedTag, 8, 0, 1, 1);

        trusted = new QLabel(SslInfo);
        trusted->setObjectName("trusted");
        trusted->setText(QString::fromUtf8("Maybe... no."));
        trusted->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayout->addWidget(trusted, 8, 1, 1, 1);

        validityPeriodTag = new QLabel(SslInfo);
        validityPeriodTag->setObjectName("validityPeriodTag");
        validityPeriodTag->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(validityPeriodTag, 9, 0, 1, 1);

        validityPeriod = new QLabel(SslInfo);
        validityPeriod->setObjectName("validityPeriod");
        validityPeriod->setText(QString::fromUtf8("August 34 2004 to Undecimber 0 2008"));
        validityPeriod->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayout->addWidget(validityPeriod, 9, 1, 1, 2);

        serialTag = new QLabel(SslInfo);
        serialTag->setObjectName("serialTag");
        serialTag->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(serialTag, 10, 0, 1, 1);

        serial = new QLabel(SslInfo);
        serial->setObjectName("serial");
        serial->setText(QString::fromUtf8("23"));
        serial->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayout->addWidget(serial, 10, 1, 1, 1);

        digestTag = new QLabel(SslInfo);
        digestTag->setObjectName("digestTag");
        digestTag->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(digestTag, 11, 0, 1, 1);

        digest = new QLabel(SslInfo);
        digest->setObjectName("digest");
        digest->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayout->addWidget(digest, 11, 1, 1, 1);

        sha1DigestTag = new QLabel(SslInfo);
        sha1DigestTag->setObjectName("sha1DigestTag");
        sha1DigestTag->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(sha1DigestTag, 12, 0, 1, 1);

        sha1Digest = new QLabel(SslInfo);
        sha1Digest->setObjectName("sha1Digest");
        sha1Digest->setText(QString::fromUtf8("B4:DB:00:2E"));
        sha1Digest->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayout->addWidget(sha1Digest, 12, 1, 1, 1);


        retranslateUi(SslInfo);

        QMetaObject::connectSlotsByName(SslInfo);
    } // setupUi

    void retranslateUi(QWidget *SslInfo)
    {
        encryptionIndicator->setText(tr2i18n("[padlock]", nullptr));
        addressTag->setText(tr2i18n("Address:", "Web page address"));
        ipTag->setText(tr2i18n("IP address:", nullptr));
        encryptionTag->setText(tr2i18n("Encryption:", nullptr));
        detailsTag->setText(tr2i18n("Details:", nullptr));
        sslVersionTag->setText(tr2i18n("SSL version:", nullptr));
        certSelectorTag->setText(tr2i18n("Certificate chain:", nullptr));
        trustedTag->setText(tr2i18n("Trusted:", nullptr));
        validityPeriodTag->setText(tr2i18n("Validity period:", nullptr));
        serialTag->setText(tr2i18n("Serial number:", nullptr));
        digestTag->setText(tr2i18n("MD5 digest:", nullptr));
        digest->setText(QString());
        sha1DigestTag->setText(tr2i18n("SHA1 digest:", nullptr));
        (void)SslInfo;
    } // retranslateUi

};

namespace Ui {
    class SslInfo: public Ui_SslInfo {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SSLINFO_H

