/********************************************************************************
** Form generated from reading UI file 'checksumswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CHECKSUMSWIDGET_H
#define UI_CHECKSUMSWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kseparator.h"

QT_BEGIN_NAMESPACE

class Ui_ChecksumsWidget
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QLineEdit *lineEdit;
    QPushButton *pasteButton;
    QLabel *feedbackLabel;
    KSeparator *kseparator;
    QWidget *calculateWidget;
    QFormLayout *formLayout;
    QLabel *md5Label;
    QLabel *sha1Label;
    QLabel *sha256Label;
    QLabel *sha512Label;
    QHBoxLayout *horizontalLayout_1;
    QPushButton *md5Button;
    QSpacerItem *horizontalSpacerMd5;
    QLineEdit *md5LineEdit;
    QPushButton *md5CopyButton;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *sha1Button;
    QSpacerItem *horizontalSpacerSha1;
    QLineEdit *sha1LineEdit;
    QPushButton *sha1CopyButton;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *sha256Button;
    QSpacerItem *horizontalSpacerSha256;
    QLineEdit *sha256LineEdit;
    QPushButton *sha256CopyButton;
    QHBoxLayout *horizontalLayout_4;
    QPushButton *sha512Button;
    QSpacerItem *horizontalSpacerSha512;
    QLineEdit *sha512LineEdit;
    QPushButton *sha512CopyButton;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *ChecksumsWidget)
    {
        if (ChecksumsWidget->objectName().isEmpty())
            ChecksumsWidget->setObjectName("ChecksumsWidget");
        ChecksumsWidget->resize(485, 463);
        verticalLayout = new QVBoxLayout(ChecksumsWidget);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(ChecksumsWidget);
        label->setObjectName("label");
        label->setWordWrap(true);
        label->setTextInteractionFlags(Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard);

        verticalLayout->addWidget(label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        lineEdit = new QLineEdit(ChecksumsWidget);
        lineEdit->setObjectName("lineEdit");

        horizontalLayout->addWidget(lineEdit);

        pasteButton = new QPushButton(ChecksumsWidget);
        pasteButton->setObjectName("pasteButton");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("edit-paste")));
        pasteButton->setIcon(icon);

        horizontalLayout->addWidget(pasteButton);


        verticalLayout->addLayout(horizontalLayout);

        feedbackLabel = new QLabel(ChecksumsWidget);
        feedbackLabel->setObjectName("feedbackLabel");
        feedbackLabel->setText(QString::fromUtf8("Feedback"));
        feedbackLabel->setTextInteractionFlags(Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard);

        verticalLayout->addWidget(feedbackLabel);

        kseparator = new KSeparator(ChecksumsWidget);
        kseparator->setObjectName("kseparator");

        verticalLayout->addWidget(kseparator);

        calculateWidget = new QWidget(ChecksumsWidget);
        calculateWidget->setObjectName("calculateWidget");
        formLayout = new QFormLayout(calculateWidget);
        formLayout->setObjectName("formLayout");
        md5Label = new QLabel(calculateWidget);
        md5Label->setObjectName("md5Label");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, md5Label);

        sha1Label = new QLabel(calculateWidget);
        sha1Label->setObjectName("sha1Label");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, sha1Label);

        sha256Label = new QLabel(calculateWidget);
        sha256Label->setObjectName("sha256Label");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, sha256Label);

        sha512Label = new QLabel(calculateWidget);
        sha512Label->setObjectName("sha512Label");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, sha512Label);

        horizontalLayout_1 = new QHBoxLayout();
        horizontalLayout_1->setObjectName("horizontalLayout_1");
        md5Button = new QPushButton(calculateWidget);
        md5Button->setObjectName("md5Button");

        horizontalLayout_1->addWidget(md5Button);

        horizontalSpacerMd5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_1->addItem(horizontalSpacerMd5);

        md5LineEdit = new QLineEdit(calculateWidget);
        md5LineEdit->setObjectName("md5LineEdit");
        md5LineEdit->setReadOnly(true);
        md5LineEdit->setVisible(false);

        horizontalLayout_1->addWidget(md5LineEdit);

        md5CopyButton = new QPushButton(calculateWidget);
        md5CopyButton->setObjectName("md5CopyButton");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("edit-copy")));
        md5CopyButton->setIcon(icon1);

        horizontalLayout_1->addWidget(md5CopyButton);


        formLayout->setLayout(0, QFormLayout::ItemRole::FieldRole, horizontalLayout_1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        sha1Button = new QPushButton(calculateWidget);
        sha1Button->setObjectName("sha1Button");

        horizontalLayout_2->addWidget(sha1Button);

        horizontalSpacerSha1 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacerSha1);

        sha1LineEdit = new QLineEdit(calculateWidget);
        sha1LineEdit->setObjectName("sha1LineEdit");
        sha1LineEdit->setReadOnly(true);
        sha1LineEdit->setVisible(false);

        horizontalLayout_2->addWidget(sha1LineEdit);

        sha1CopyButton = new QPushButton(calculateWidget);
        sha1CopyButton->setObjectName("sha1CopyButton");
        sha1CopyButton->setIcon(icon1);

        horizontalLayout_2->addWidget(sha1CopyButton);


        formLayout->setLayout(1, QFormLayout::ItemRole::FieldRole, horizontalLayout_2);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        sha256Button = new QPushButton(calculateWidget);
        sha256Button->setObjectName("sha256Button");

        horizontalLayout_3->addWidget(sha256Button);

        horizontalSpacerSha256 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacerSha256);

        sha256LineEdit = new QLineEdit(calculateWidget);
        sha256LineEdit->setObjectName("sha256LineEdit");
        sha256LineEdit->setReadOnly(true);
        sha256LineEdit->setVisible(false);

        horizontalLayout_3->addWidget(sha256LineEdit);

        sha256CopyButton = new QPushButton(calculateWidget);
        sha256CopyButton->setObjectName("sha256CopyButton");
        sha256CopyButton->setIcon(icon1);

        horizontalLayout_3->addWidget(sha256CopyButton);


        formLayout->setLayout(2, QFormLayout::ItemRole::FieldRole, horizontalLayout_3);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        sha512Button = new QPushButton(calculateWidget);
        sha512Button->setObjectName("sha512Button");

        horizontalLayout_4->addWidget(sha512Button);

        horizontalSpacerSha512 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacerSha512);

        sha512LineEdit = new QLineEdit(calculateWidget);
        sha512LineEdit->setObjectName("sha512LineEdit");
        sha512LineEdit->setReadOnly(true);
        sha512LineEdit->setVisible(false);

        horizontalLayout_4->addWidget(sha512LineEdit);

        sha512CopyButton = new QPushButton(calculateWidget);
        sha512CopyButton->setObjectName("sha512CopyButton");
        sha512CopyButton->setIcon(icon1);

        horizontalLayout_4->addWidget(sha512CopyButton);


        formLayout->setLayout(3, QFormLayout::ItemRole::FieldRole, horizontalLayout_4);


        verticalLayout->addWidget(calculateWidget);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        md5Label->setBuddy(md5Button);
        sha1Label->setBuddy(sha1Button);
        sha256Label->setBuddy(sha256Button);
        sha512Label->setBuddy(sha512Button);
#endif // QT_CONFIG(shortcut)

        retranslateUi(ChecksumsWidget);

        QMetaObject::connectSlotsByName(ChecksumsWidget);
    } // setupUi

    void retranslateUi(QWidget *ChecksumsWidget)
    {
        label->setText(tr2i18n("Copy and paste a checksum in the field below.<br/>A checksum is usually provided by the website you downloaded this file from.", nullptr));
        lineEdit->setPlaceholderText(tr2i18n("Expected checksum (MD5, SHA1, SHA256 or SHA512)...", nullptr));
#if QT_CONFIG(tooltip)
        pasteButton->setToolTip(tr2i18n("Click to paste the checksum from the clipboard to the input field.", nullptr));
#endif // QT_CONFIG(tooltip)
        pasteButton->setText(tr2i18n("Paste", nullptr));
        md5Label->setText(tr2i18n("MD5:", nullptr));
        sha1Label->setText(tr2i18n("SHA1:", nullptr));
        sha256Label->setText(tr2i18n("SHA256:", nullptr));
        sha512Label->setText(tr2i18n("SHA512:", nullptr));
        md5Button->setText(tr2i18n("Calculate", nullptr));
#if QT_CONFIG(tooltip)
        md5CopyButton->setToolTip(tr2i18n("Click to copy the checksum to the clipboard.", nullptr));
#endif // QT_CONFIG(tooltip)
        md5CopyButton->setText(tr2i18n("Copy", nullptr));
        sha1Button->setText(tr2i18n("Calculate", nullptr));
#if QT_CONFIG(tooltip)
        sha1CopyButton->setToolTip(tr2i18n("Click to copy the checksum to the clipboard.", nullptr));
#endif // QT_CONFIG(tooltip)
        sha1CopyButton->setText(tr2i18n("Copy", nullptr));
        sha256Button->setText(tr2i18n("Calculate", nullptr));
#if QT_CONFIG(tooltip)
        sha256CopyButton->setToolTip(tr2i18n("Click to copy the checksum to the clipboard.", nullptr));
#endif // QT_CONFIG(tooltip)
        sha256CopyButton->setText(tr2i18n("Copy", nullptr));
        sha512Button->setText(tr2i18n("Calculate", nullptr));
#if QT_CONFIG(tooltip)
        sha512CopyButton->setToolTip(tr2i18n("Click to copy the checksum to the clipboard.", nullptr));
#endif // QT_CONFIG(tooltip)
        sha512CopyButton->setText(tr2i18n("Copy", nullptr));
        (void)ChecksumsWidget;
    } // retranslateUi

};

namespace Ui {
    class ChecksumsWidget: public Ui_ChecksumsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CHECKSUMSWIDGET_H

