/****************************************************************************
** Meta object code from reading C++ file 'kcoreurlnavigator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-6.28.0/src/gui/kcoreurlnavigator.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kcoreurlnavigator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17KCoreUrlNavigatorE_t {};
} // unnamed namespace

template <> constexpr inline auto KCoreUrlNavigator::qt_create_metaobjectdata<qt_meta_tag_ZN17KCoreUrlNavigatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KCoreUrlNavigator",
        "currentLocationUrlChanged",
        "",
        "currentUrlAboutToChange",
        "QUrl",
        "newUrl",
        "historySizeChanged",
        "urlSelectionRequested",
        "url",
        "historyIndexChanged",
        "historyChanged",
        "locationUrl",
        "historyIndex",
        "saveLocationState",
        "QVariant",
        "state",
        "locationState",
        "goBack",
        "goForward",
        "goUp",
        "currentLocationUrl",
        "historySize"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentLocationUrlChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentUrlAboutToChange'
        QtMocHelpers::SignalData<void(const QUrl &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'historySizeChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'urlSelectionRequested'
        QtMocHelpers::SignalData<void(const QUrl &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 8 },
        }}),
        // Signal 'historyIndexChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'historyChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'locationUrl'
        QtMocHelpers::MethodData<QUrl(int) const>(11, 2, QMC::AccessPublic, 0x80000000 | 4, {{
            { QMetaType::Int, 12 },
        }}),
        // Method 'locationUrl'
        QtMocHelpers::MethodData<QUrl() const>(11, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 4),
        // Method 'saveLocationState'
        QtMocHelpers::MethodData<void(const QVariant &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Method 'locationState'
        QtMocHelpers::MethodData<QVariant(int) const>(16, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::Int, 12 },
        }}),
        // Method 'locationState'
        QtMocHelpers::MethodData<QVariant() const>(16, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 14),
        // Method 'goBack'
        QtMocHelpers::MethodData<bool()>(17, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'goForward'
        QtMocHelpers::MethodData<bool()>(18, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'goUp'
        QtMocHelpers::MethodData<bool()>(19, 2, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'currentLocationUrl'
        QtMocHelpers::PropertyData<QUrl>(20, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'historySize'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags, 2),
        // property 'historyIndex'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KCoreUrlNavigator, qt_meta_tag_ZN17KCoreUrlNavigatorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KCoreUrlNavigator::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KCoreUrlNavigatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KCoreUrlNavigatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17KCoreUrlNavigatorE_t>.metaTypes,
    nullptr
} };

void KCoreUrlNavigator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KCoreUrlNavigator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentLocationUrlChanged(); break;
        case 1: _t->currentUrlAboutToChange((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 2: _t->historySizeChanged(); break;
        case 3: _t->urlSelectionRequested((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 4: _t->historyIndexChanged(); break;
        case 5: _t->historyChanged(); break;
        case 6: { QUrl _r = _t->locationUrl((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 7: { QUrl _r = _t->locationUrl();
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->saveLocationState((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 9: { QVariant _r = _t->locationState((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 10: { QVariant _r = _t->locationState();
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->goBack();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->goForward();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->goUp();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KCoreUrlNavigator::*)()>(_a, &KCoreUrlNavigator::currentLocationUrlChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreUrlNavigator::*)(const QUrl & )>(_a, &KCoreUrlNavigator::currentUrlAboutToChange, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreUrlNavigator::*)()>(_a, &KCoreUrlNavigator::historySizeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreUrlNavigator::*)(const QUrl & )>(_a, &KCoreUrlNavigator::urlSelectionRequested, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreUrlNavigator::*)()>(_a, &KCoreUrlNavigator::historyIndexChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCoreUrlNavigator::*)()>(_a, &KCoreUrlNavigator::historyChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->currentLocationUrl(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->historySize(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->historyIndex(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCurrentLocationUrl(*reinterpret_cast<QUrl*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KCoreUrlNavigator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KCoreUrlNavigator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KCoreUrlNavigatorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KCoreUrlNavigator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void KCoreUrlNavigator::currentLocationUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KCoreUrlNavigator::currentUrlAboutToChange(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KCoreUrlNavigator::historySizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KCoreUrlNavigator::urlSelectionRequested(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KCoreUrlNavigator::historyIndexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KCoreUrlNavigator::historyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
